/*************************************************
FOX-SX18 2 FLEX analog

Esempio di collegamento tra FOX e la scheda FLEX analog.
TEST A/D e TEMPERATURA

Collegamenti elettrici:
  PB7 (pin 37 FOX) --> SCK (Pin 2 EXP IN sulla FLEX)
  PB6 (pin 38 FOX) --> SDA (Pin 1 EXP IN sulla FLEX)
  VCC +5V (pin 12 FOX )  --  VCC (Pin 9 EXP IN sulla FLEX) 
  GND (pin 40 FOX )  --  GND (Pin 10 EXP IN sulla FLEX)
  
  SX18 - FLEX analog
  EXPANDER SX16    ---  EXPANDER IN

****************************************************/

#include "stdlib.h"
#include "stdio.h"     
#include "unistd.h"    
#include "sys/ioctl.h"
#include "fcntl.h"     
#include "time.h"     
#include "asm/etraxgpio.h"

#define I2C_DATA_LINE       1<<6
#define I2C_CLOCK_LINE      1<<7

#define COSTANTE            (float)5/255
#define AD1_ADDRESS         0x00  //S2 - A1 e A2 chiusi
#define AD2_ADDRESS         0x01
#define TEMP_ADDRESS        0x03  //JP6 - A2 chiuso
#ifndef IO_SETGET_INPUT
  #define IO_SETGET_INPUT   0x12
#endif
#ifndef IO_SETGET_OUTPUT
  #define IO_SETGET_OUTPUT  0x13
#endif

int i2c_fd;
//-----------------------------
//FUNZIONI
//-----------------------------
// Software delay in us
void udelay(int us) {
  int a;
  int b;
  int delayvar=1111;

  for (b=0;b<33;b++) {
    for (a=0;a<us;a++) {
      delayvar*=3;
      delayvar/=3;
    }
  }  
}   

// Get the SDA line state
int i2c_getbit(void) {
  unsigned int value;
  value=ioctl(i2c_fd, _IO(ETRAXGPIO_IOCTYPE, IO_READBITS));
  if ((value&(I2C_DATA_LINE))==0) 
    return 0;
  else 
    return 1;
}

// Set the SDA line state
void i2c_data(int state) {
  if (state==1) 
    ioctl(i2c_fd, _IO(ETRAXGPIO_IOCTYPE, IO_SETBITS), I2C_DATA_LINE);
  else 
    ioctl(i2c_fd, _IO(ETRAXGPIO_IOCTYPE, IO_CLRBITS), I2C_DATA_LINE);
}

// Set the SCL line state
void i2c_clk(int state) {
  if (state==1) 
    ioctl(i2c_fd, _IO(ETRAXGPIO_IOCTYPE, IO_SETBITS), I2C_CLOCK_LINE);
  else 
    ioctl(i2c_fd, _IO(ETRAXGPIO_IOCTYPE, IO_CLRBITS), I2C_CLOCK_LINE);
}

// Set the SDA line as output
void i2c_dir_out(void) {
  int iomask;
  iomask = I2C_DATA_LINE;
  ioctl(i2c_fd, _IO(ETRAXGPIO_IOCTYPE, IO_SETGET_OUTPUT), &iomask);
}

// Set the SDA line as input
void i2c_dir_in(void) {
  int iomask;
  iomask = I2C_DATA_LINE;
  ioctl(i2c_fd, _IO(ETRAXGPIO_IOCTYPE, IO_SETGET_INPUT), &iomask);
}

// Open the GPIOB dev 
int i2c_open(void) {
  i2c_fd = open("/dev/gpiob", O_RDWR);
  i2c_data(1);
  i2c_dir_out();
  i2c_clk(1);
  i2c_data(1);
  udelay(100);
  return i2c_fd;
}

// Close the GPIOB dev 
void i2c_close(void) {
  close(i2c_fd);
}

// Read a byte from I2C bus and send the ack sequence
unsigned char i2c_inbyte(int ack) {
  unsigned char value = 0;
  int bitvalue;
  int i;

  // Read data byte
  i2c_dir_in();
  for (i=0;i<8;i++) {
    i2c_clk(1);
    udelay(5);
    bitvalue = i2c_getbit();
    value |= bitvalue;
    if (i<7) value <<= 1;
    i2c_clk(0);
    udelay(5);
  }
  // Send Ack
  if(ack){
  i2c_dir_out();
  i2c_data(0);
  i2c_clk(1);
  udelay(5);
  i2c_clk(0);
  }
  udelay(200);	
  return value;
}

// Send a start sequence to I2C bus
void i2c_start(void){
  i2c_dir_out();
  i2c_clk(1);
  i2c_data(1);
  udelay(5);
  i2c_data(0);
}

// Send a stop sequence to I2C bus
void i2c_stop(void) {
  i2c_dir_out();
  i2c_clk(1);
  i2c_data(0);
  udelay(5);
  i2c_data(1);
}

// Send a byte to the I2C bus and return the ack sequence from slave
int i2c_outbyte(unsigned char x) {
  int i;
  int ack;

  i2c_clk(0);
  for (i=0;i<8;i++) {
    if (x & 0x80) 
      i2c_data(1);
    else
      i2c_data(0);
    i2c_clk(1);
    udelay(5);
    i2c_clk(0);
    udelay(5);
    x <<= 1;
  }
  i2c_data(0);
  i2c_dir_in();
  i2c_clk(1);
  ack=i2c_getbit();
  i2c_clk(0);
  i2c_dir_out();
  if (ack==0)
    return 1;
  else 
    return 0;
}

//FUNZIONE DI LETTURA DELL'AD
int read_PCF8591(unsigned char address, unsigned char *buf) {
  i2c_stop();
  i2c_start();
  if (i2c_outbyte(0x90 | (address<<1))==0) {
    i2c_stop();
    printf("Ricevuto NACK 1\n");
    return -1;
  }
  if (i2c_outbyte(0x04)==0) {
    i2c_stop();
    printf("Ricevuto NACK 2\n");
    return -1;
  }
  i2c_stop();
  udelay(100);
  i2c_start();
  if (i2c_outbyte(0x91 | (address<<1))==0) {
    printf("Ricevuto NACK 3\n");
  }
  i2c_inbyte(1);
  buf[0]=i2c_inbyte(1); 
  buf[1]=i2c_inbyte(1);
  buf[2]=i2c_inbyte(1);
  buf[3]=i2c_inbyte(0);
  i2c_stop();
  return 1;
}

//FUNZIONE D'INIZIALIZZAZIONE SENSORE DI TEMPERATURA
int DS1621Init(unsigned char address){
  i2c_start();
  if (i2c_outbyte(0x90 + (address<<1))==0) {
    printf("Ricevuto NACK 4\n");
  }
  if (i2c_outbyte(0xAC)==0) {
    printf("Ricevuto NACK 5\n");
  }
  if (i2c_outbyte(0x02)==0) {
    printf("Ricevuto NACK 6\n");
  }
  i2c_stop();
  //Avvia la conversione
  i2c_start();
  if (i2c_outbyte(0x90 + (address<<1))==0) {
    printf("Ricevuto NACK 7\n");
  }
  if (i2c_outbyte(0xEE)==0) {
    printf("Ricevuto NACK 8\n");
  }
  i2c_stop();
}
//FUNZIONE DI LETTURA DEL SENSORE DI TEMPERATURA
float DS1621Read(unsigned char address) {
  unsigned char value = 0;
  auto float temp=0;

  i2c_stop();
  //Lettura ultima acquisizione
  i2c_start();
  if (i2c_outbyte(0x90 + (address<<1))==0) {
    printf("Ricevuto NACK 9\n");
  }
  if (i2c_outbyte(0xAA)==0) {
    printf("Ricevuto NACK 10\n");
  }
  i2c_stop();
  i2c_start();
  if (i2c_outbyte(0x91 + (address<<1))==0) {
    printf("Ricevuto NACK 11\n");
  }
  value=i2c_inbyte(1); 
  if (value & 0x80) {
    temp+=(!value)+1;
    temp=-temp;
  } else {
    // Temperatura >0
    temp+=value;
  }
  value=i2c_inbyte(1); 
  if (value==0x80) temp+=0.5;
  i2c_stop();
  i2c_start();
  i2c_outbyte(0x91 + (address<<1));
  i2c_stop();

  return (temp);
}

//------------------------------
//
//PROGRAMMA PRINCIPALE
//
//------------------------------
int main(void) {
  unsigned char buffer[4];
  if (i2c_open()<0) {
    printf("Apertura del bus I2C fallita\n");
    return 1;
  }
  //Inizializza il sensore di temperatura
  DS1621Init(TEMP_ADDRESS);
  sleep(1);
  //Ciclo infinito di lettura AD e tempetatura
  while (1) {
    system("clear");
    printf("---------------------------------------\n");
    printf(" FOX 2 FLEX Analog\n\n");
    printf(" Ver: 1.0\n");
    printf(" Author: Daniele De Santis\n");
    printf(" E-Mail: desantis@areasx.com\n\n");
    printf(" Programma di test per SX18 e FLEX analog  \n");
    printf("---------------------------------------\n\n");
    printf ("Temperatura:  %.1f\n\n", DS1621Read(TEMP_ADDRESS));

    printf("Lettura A/D 1\n");
    read_PCF8591(AD1_ADDRESS, buffer);
    printf("CH 1 = %.2fv\n",buffer[0]*COSTANTE);
    printf("CH 2 = %.2fv\n",buffer[1]*COSTANTE);
    printf("CH 3 = %.2fv\n",buffer[2]*COSTANTE);
    printf("CH 4 = %.2fv\n",buffer[3]*COSTANTE);
    printf("\nLettura A/D 2\n");
    read_PCF8591(AD2_ADDRESS, buffer);
    printf("CH 1 = %.2fv\n",buffer[0]*COSTANTE);
    printf("CH 2 = %.2fv\n",buffer[1]*COSTANTE);
    printf("CH 3 = %.2fv\n",buffer[2]*COSTANTE);
    printf("CH 4 = %.2fv\n",buffer[3]*COSTANTE);
    
    printf("--------------------\n");
    sleep(1);
  }
  i2c_close();
  return 0;
}
